@@ -5,7 +5,6 @@ Created on Thu Oct 18 19:24:05 2018
 @author: merzbach
 
 """
-
 from copy import deepcopy
 from datetime import datetime
 from functools import wraps
@@ -14,6 +13,7 @@ try:
 except:
     pass
 import imageio
+import json
 import numpy as np
 import os
 import sys
@@ -57,7 +57,7 @@ try:
 except ModuleNotFoundError:
     Tensor = type(None)
 
-from pysmtb.utils import crop_bounds, pad
+from pysmtb.utils import crop_bounds, pad, collage
 
 '''
 def MyPyQtSlot(*args):
@@ -188,6 +188,7 @@ class IV(QMainWindow):
         self.autoscaleUpper = self.autoscaleEnabled
         self.autoscaleGlobal = kwargs.get('autoscaleGlobal', False)
         self.collageActive = kwargs.get('collage', False)
+        self.collage_tight = kwargs.get('collageTight', kwargs.get('collage_tight', True))
         self.collageTranspose = kwargs.get('collageTranspose', False)
         self.collageTransposeIms = kwargs.get('collageTransposeIms', False)
         if 'nr' in kwargs and 'nc' in kwargs:
@@ -206,9 +207,12 @@ class IV(QMainWindow):
         self.collage_nc = nc
         self.collage_border_width = kwargs.get('collageBorderWidth', 0)
         self.collage_border_value = kwargs.get('collageBorderValue', 0.)
+        self.has_alpha = kwargs.get('has_alpha', False)
+        self.blend_alpha = kwargs.get('blend_alpha', False)
+        self.background = kwargs.get('background', 0.0)
         self.crop = kwargs.get('crop', False)
         self.crop_global = kwargs.get('crop_global', True)
-        self.crop_background = kwargs.get('crop_background', 0)
+        self.crop_background = kwargs.get('crop_background', 0.0)
         self.zoom_factor = 1.1
         self.x_zoom = True
         self.y_zoom = True
@@ -218,7 +222,8 @@ class IV(QMainWindow):
         self.annotate_numbers = kwargs.get('annotate_numbers', True)
         self.font_size = kwargs.get('font_size', 12)
         self.font_color = kwargs.get('font_color', 1)
-        self.labels = kwargs.get('labels', None)
+        if len(self.labels) == 0:
+            self.labels = None
         if self.labels is not None:
             assert len(self.labels) == len(self.images), 'number of labels %d must match number of images %d'\
                                                          % (len(self.labels), len(self.images))
@@ -227,8 +232,12 @@ class IV(QMainWindow):
         # TODO: expose these
         self.spec_wl0 = 380
         self.spec_wl1 = 730
-        self.spec_cmf_names = list(colour.MSDS_CMFS.keys())
-        self.spec_illuminant_names = list(colour.SDS_ILLUMINANTS.keys())
+        if colour is not None:
+            self.spec_cmf_names = list(colour.MSDS_CMFS.keys())
+            self.spec_illuminant_names = list(colour.SDS_ILLUMINANTS.keys())
+        else:
+            self.spec_cmf_names = ['pip install colour-science']
+            self.spec_illuminant_names = ['pip install colour-science']
         self.spec_cmf_selected_name = 'CIE 1931 2 Degree Standard Observer'
         self.spec_illuminant_selected_name = 'E'
 
@@ -238,6 +247,7 @@ class IV(QMainWindow):
         self.xmaxs = []
         self.ymins = []
         self.ymaxs = []
+        self.overlay_ths = []
         self._compute_crop_bounds()
         self._init_ui()
 
@@ -324,15 +334,19 @@ class IV(QMainWindow):
         self.uiCBAutoscaleGlobal = _add_widget(width // 2, QCheckBox, 'global', 'stateChanged', self._callback_check_box, self.autoscaleGlobal)
         if self.nims > 1:
             self.uiCBCollageActive = _add_widget(width // 2, QCheckBox, 'enable', 'stateChanged', self._callback_check_box, self.collageActive)
+            self.uiCBCollageTight = _add_widget(width // 2, QCheckBox, 'tight', 'stateChanged', self._callback_check_box, self.collage_tight)
             self.uiCBCollageTranspose = _add_widget(width // 2, QCheckBox, 'transp.', 'stateChanged', self._callback_check_box, self.collageTranspose)
             self.uiCBCollageTransposeIms = _add_widget(width // 2, QCheckBox, 'transp. ims.', 'stateChanged', self._callback_check_box, self.collageTransposeIms)
             self.uiLECollageNr = _add_widget(width // 2, QLineEdit, None, 'editingFinished', self._callback_line_edit, self.collage_nr)
             self.uiLECollageNc = _add_widget(width // 2, QLineEdit, None, 'editingFinished', self._callback_line_edit, self.collage_nc)
             self.uiLECollageBW = _add_widget(width // 2, QLineEdit, None, 'editingFinished', self._callback_line_edit, self.collage_border_width)
             self.uiLECollageBV = _add_widget(width // 2, QLineEdit, None, 'editingFinished', self._callback_line_edit, self.collage_border_value)
+        self.uiCBHasAlpha = _add_widget(width // 2, QCheckBox, 'available', 'stateChanged', self._callback_check_box, self.has_alpha)
+        self.uiCBBlendAlpha = _add_widget(width // 2, QCheckBox, 'blend', 'stateChanged', self._callback_check_box, self.blend_alpha)
         self.uiCBCrop = _add_widget(width // 2, QCheckBox, 'enable', 'stateChanged', self._callback_check_box, self.crop)
-        self.uiCBCropGlobal = _add_widget(width // 2, QCheckBox, 'enable', 'stateChanged', self._callback_check_box, self.crop_global)
+        self.uiCBCropGlobal = _add_widget(width // 2, QCheckBox, 'global', 'stateChanged', self._callback_check_box, self.crop_global)
         self.uiLECropBackground = _add_widget(width // 2, QLineEdit, None, 'editingFinished', self._callback_line_edit, self.crop_background)
+        self.uiLEBackground = _add_widget(width // 2, QLineEdit, None, 'editingFinished', self._callback_line_edit, self.background)
         self.uiCBAnnotate = _add_widget(width // 2, QCheckBox, 'enable', 'stateChanged', self._callback_check_box, self.annotate)
         self.uiCBAnnotateNumbers = _add_widget(width // 2, QCheckBox, 'numbers', 'stateChanged', self._callback_check_box, self.annotate_numbers)
         self.uiLEFontSize = _add_widget(width, QLineEdit, None, 'editingFinished', self._callback_line_edit, self.font_size)
@@ -340,9 +354,11 @@ class IV(QMainWindow):
 
         # add spectral to RGB conversion options
         self.uiCBSpecCMFs = _add_widget(width, QComboBox, None, 'activated', self._callback_combobox, self.spec_cmf_names)
-        self.uiCBSpecCMFs.setCurrentIndex(np.where([name == self.spec_cmf_selected_name for name in self.spec_cmf_names])[0][0])
+        cmf_ind = np.where([name == self.spec_cmf_selected_name for name in self.spec_cmf_names])[0]
+        self.uiCBSpecCMFs.setCurrentIndex(cmf_ind[0] if len(cmf_ind) else 0)
         self.uiCBSpecIlluminants = _add_widget(width, QComboBox, None, 'activated', self._callback_combobox, self.spec_illuminant_names)
-        self.uiCBSpecIlluminants.setCurrentIndex(np.where([name == self.spec_illuminant_selected_name for name in self.spec_illuminant_names])[0][0])
+        illum_ind = np.where([name == self.spec_illuminant_selected_name for name in self.spec_illuminant_names])[0]
+        self.uiCBSpecIlluminants.setCurrentIndex(illum_ind[0] if len(illum_ind) else 0)
         self.uiLESpecWL0 = _add_widget(width // 2, QLineEdit, None, 'editingFinished', self._callback_line_edit, self.spec_wl0)
         self.uiLESpecWL1 = _add_widget(width // 2, QLineEdit, None, 'editingFinished', self._callback_line_edit, self.spec_wl1)
 
@@ -389,13 +405,15 @@ class IV(QMainWindow):
         _add_row(QLabel('bounds:'), _multicolumn(self.uiLabelAutoscaleLower, self.uiLabelAutoscaleUpper))
         _add_row(_hdiv())
         if self.nims > 1:
-            _add_row(QLabel('collage:'), _multicolumn(self.uiCBCollageActive, self.uiCBCollageTranspose))
-            _add_row(QLabel('per img:'), self.uiCBCollageTransposeIms)
+            _add_row(QLabel('collage:'), _multicolumn(self.uiCBCollageActive, self.uiCBCollageTight))
+            _add_row(QLabel('per img:'), _multicolumn(self.uiCBCollageTranspose, self.uiCBCollageTransposeIms))
             _add_row(QLabel('NR x NC:'), _multicolumn(self.uiLECollageNr, self.uiLECollageNc))
             _add_row(QLabel('BW x BV:'), _multicolumn(self.uiLECollageBW, self.uiLECollageBV))
             _add_row(_hdiv())
+        _add_row(QLabel('alpha:'), _multicolumn(self.uiCBHasAlpha, self.uiCBBlendAlpha))
+        _add_row(_hdiv())
         _add_row(QLabel('crop:'), _multicolumn(self.uiCBCrop, self.uiCBCropGlobal))
-        _add_row(QLabel('crop bgrnd:'), self.uiLECropBackground)
+        _add_row(QLabel('crop bgrnd:'), _multicolumn(self.uiLECropBackground, self.uiLEBackground))
         _add_row(_hdiv())
         _add_row(QLabel('annotate:'), _multicolumn(self.uiCBAnnotate, self.uiCBAnnotateNumbers))
         _add_row(QLabel('font size:'), self.uiLEFontSize)
@@ -470,8 +488,17 @@ class IV(QMainWindow):
         QShortcut(QKeySequence('Shift+a'), self.widget).activated.connect(self._toggle_autoscale_use_prctiles)
 
     def _callback_line_edit(self, ui, *args):
+        tmp = ui.text()
         try:
-            tmp = float(ui.text())
+            try:
+                tmp = json.loads(tmp)
+                if isinstance(tmp, list):
+                    tmp = np.array(tmp)
+            except json.decoder.JSONDecodeError:
+                try:
+                    tmp = float(tmp)
+                except:
+                    return
         except:
             return
         
@@ -489,9 +516,25 @@ class IV(QMainWindow):
             self.autoscale()
         elif hasattr(self, 'uiLECollageNr') and ui == self.uiLECollageNr:
             self.collage_nr = int(tmp)
+            if self.nims > self.collage_nc * self.collage_nr:
+                # increase nc to match selected nr given nims
+                self.collage_nc = int(np.ceil(self.nims / self.collage_nr))
+                self.uiLECollageNc.blockSignals(True)
+                self.uiLECollageNr.blockSignals(True)
+                self.uiLECollageNc.setText(str(self.collage_nc))
+                self.uiLECollageNr.blockSignals(False)
+                self.uiLECollageNc.blockSignals(False)
             self._display_collage()
         elif hasattr(self, 'uiLECollageNc') and ui == self.uiLECollageNc:
             self.collage_nc = int(tmp)
+            if self.nims > self.collage_nc * self.collage_nr:
+                # increase nr to match selected nc given nims
+                self.collage_nr = int(np.ceil(self.nims / self.collage_nc))
+                self.uiLECollageNc.blockSignals(True)
+                self.uiLECollageNr.blockSignals(True)
+                self.uiLECollageNr.setText(str(self.collage_nr))
+                self.uiLECollageNr.blockSignals(False)
+                self.uiLECollageNc.blockSignals(False)
             self._display_collage()
         elif hasattr(self, 'uiLECollageBW') and ui == self.uiLECollageBW:
             self.collage_border_width = int(tmp)
@@ -509,6 +552,9 @@ class IV(QMainWindow):
             self.crop_background = tmp
             self._compute_crop_bounds()
             self._display_image()
+        elif ui == self.uiLEBackground:
+            self.background = tmp
+            self._display_image()
         elif hasattr(self, 'uiLESpecWL0') and ui == self.uiLESpecWL0:
             self.spec_wl0 = tmp
             self._display_image()
@@ -539,12 +585,21 @@ class IV(QMainWindow):
         elif hasattr(self, 'uiCBCollageActive') and ui == self.uiCBCollageActive:
             self.collageActive = bool(state)
             self._display_image()
+        elif hasattr(self, 'uiCBCollageTight') and ui == self.uiCBCollageTight:
+            self.collage_tight = bool(state)
+            self._display_image()
         elif hasattr(self, 'uiCBCollageTranspose') and ui == self.uiCBCollageTranspose:
             self.collageTranspose = bool(state)
             self._display_image()
         elif hasattr(self, 'uiCBCollageTransposeIms') and ui == self.uiCBCollageTransposeIms:
             self.collageTransposeIms = bool(state)
             self._display_image()
+        elif ui == self.uiCBHasAlpha:
+            self.has_alpha = bool(state)
+            self._display_image()
+        elif ui == self.uiCBBlendAlpha:
+            self.blend_alpha = bool(state)
+            self._display_image()
         elif ui == self.uiCBCrop:
             self.crop = bool(state)
             self._display_image()
@@ -591,6 +646,8 @@ class IV(QMainWindow):
         im = self.images[i]
         if self.crop:
             im = im[self.ymins[i]:self.ymaxs[i], self.xmins[i]:self.xmaxs[i], :]
+        if im.dtype != np.float32:
+            im = im.astype(np.float32)
         if tonemap:
             im = self.tonemap(im)
         if decorate and self.annotate:
@@ -636,6 +693,10 @@ class IV(QMainWindow):
                 im = self.get_img(tonemap=False, decorate=False)
                 lower = np.min(im)
                 upper = np.max(im)
+        if upper == lower:
+            # enforce scaling by 1 to avoid zero division
+            lower -= 0.5
+            upper += 0.5
         if self.autoscaleLower:
             self.set_offset(lower, False)
             self.uiLabelAutoscaleLower.setText('%f' % lower)
@@ -650,6 +711,7 @@ class IV(QMainWindow):
     def _display_collage(self):
         # arrange all images in a collage and display them
         if self.collage_nr * self.collage_nc < self.nims:
+            # reset to default in case nc * nr < nims
             nc = int(np.ceil(np.sqrt(self.nims)))
             nr = int(np.ceil(self.nims / nc))
             self.collage_nr = nr
@@ -660,53 +722,17 @@ class IV(QMainWindow):
             self.uiLECollageNc.setText(str(nc))
             self.uiLECollageNr.blockSignals(False)
             self.uiLECollageNc.blockSignals(False)
-        else:
-            nc = self.collage_nc
-            nr = self.collage_nr
         
         # pad array so it matches the product nc * nr
-        padding = nc * nr - self.nims
         ims = self.get_imgs(tonemap=True, decorate=True)
-        h = np.max([im.shape[0] for im in ims])
-        w = np.max([im.shape[1] for im in ims])
-        num_channels = np.max([im.shape[2] for im in ims])
-        ims = [pad(im, new_width=w, new_height=h, new_num_channels=num_channels) for im in ims]
-        ims += [self.collage_border_value * np.ones((h, w, num_channels))] * padding
-        coll = np.stack(ims, axis=3)
-        coll = np.reshape(coll, (h, w, num_channels, nr, nc))
-        # 0  1  2   3   4
-        # y, x, ch, ro, co
-        if self.collage_border_width:
-            # pad each patch by border if requested
-            coll = np.append(coll, self.collage_border_value * np.ones((self.collage_border_width, ) + coll.shape[1 : 5]), axis=0)
-            coll = np.append(coll, self.collage_border_value * np.ones((coll.shape[0], self.collage_border_width) + coll.shape[2 : 5]), axis=1)
-        if self.collageTranspose:
-            nim0 = nc
-            nim1 = nr
-            if self.collageTransposeIms:
-                dim0 = w
-                dim1 = h
-                #                          nr w  nc h  ch
-                coll = np.transpose(coll, (4, 1, 3, 0, 2))
-            else:
-                dim0 = h
-                dim1 = w
-                #                          nr h  nc w  ch
-                coll = np.transpose(coll, (4, 0, 3, 1, 2))
-        else:
-            nim0 = nr
-            nim1 = nc
-            if self.collageTransposeIms:
-                dim0 = w
-                dim1 = h
-                #                          nc w  nr h  ch
-                coll = np.transpose(coll, (3, 1, 4, 0, 2))
-            else:
-                dim0 = h
-                dim1 = w
-                #                          nc h  nr w  ch
-                coll = np.transpose(coll, (3, 0, 4, 1, 2))
-        coll = np.reshape(coll, ((dim0 + self.collage_border_width) * nim0, (dim1 + self.collage_border_width) * nim1, num_channels))
+        coll = collage(images=ims,
+                       nc=self.collage_nc,
+                       nr=self.collage_nr,
+                       tight=self.collage_tight,
+                       transpose=self.collageTranspose,
+                       transpose_ims=self.collageTransposeIms,
+                       bv=self.collage_border_value,
+                       bw=self.collage_border_width)
 
         self.ax.clear()
         if coll.dtype == np.float16:
@@ -777,36 +803,80 @@ class IV(QMainWindow):
         for kid in kids:
             if isinstance(kid, matplotlib.text.Text):
                 kid.set_visible(False)
-        lims = self.ih.axes.axis()
-        inds = (np.array(lims) + 0.5).astype(np.int32)
         im = self.ih.get_array()
-        im = im[inds[3]: inds[2], inds[0]: inds[1], :]
+        lims = np.array(self.ih.axes.axis())
+        lims[0] = np.maximum(0, lims[0])
+        lims[1] = np.minimum(im.shape[0], lims[1])
+        lims[2] = np.minimum(im.shape[1], lims[2])
+        lims[3] = np.maximum(0, lims[3])
+        inds = (np.array(lims) + 0.5).astype(np.int32)
         xs = np.r_[inds[0]: inds[1]]
         ys = np.r_[inds[3]: inds[2]]
-
+        
+        for th in self.overlay_ths:
+            try:
+                th.remove()
+            except:
+                pass
+        ths = []
+        rgb2lum = np.r_[0.299, 0.587, 0.114]
         for xi, x0 in enumerate(xs):
             for yi, y0 in enumerate(ys):
                 pixel = im[yi, xi, :]
-                color = (pixel + 0.5) % 1.
-                if im.ndim == 3 and im.shape[2] == 3:
-                    th = self.ax.text(x0, y0 + 1.0, '% 6.3f\n% 6.3f\n% 6.3f\n' % tuple(pixel), fontsize=10, color=color)
+                if np.sum(rgb2lum * pixel) > 0.5:
+                    color = np.r_[0., 0., 0.]
                 else:
-                    th = self.ax.text(x0, y0 + 1.0, '% 6.3f' % im[0, 0], fontsize=10, color=color)
+                    color = np.r_[1., 1., 1.]
+                if im.ndim == 3 and im.shape[2] == 3:
+                    ths.append(self.ax.text(x0 - 0.5, y0 + 0.5, '% 6.3f\n% 6.3f\n% 6.3f\n' % tuple(pixel), fontsize=8, color=color))
+                else:
+                    ths.append(self.ax.text(x0 - 0.5, y0 + 0.5, '% 6.3f' % im[0, 0], fontsize=8, color=color))
+        self.overlay_ths = ths
         self.fig.canvas.draw()
 
+    def blend(self, im, alpha):
+        """perform alpha blending of input image and some user-specified background"""
+        bgrnd = np.array(self.background)
+        while bgrnd.ndim < 3:
+            bgrnd = bgrnd[None]
+        if im.shape[2] == 1 and bgrnd.shape[2] > im.shape[2]:
+            # handle intensity image with RGB background
+            im = np.repeat(im, bgrnd.shape[2], axis=2)
+        return alpha * im + (1 - alpha) * bgrnd
+
     def tonemap(self, im):
         """apply simple scaling & gamma based tonemapping to HDR image, convert spectral to RGB"""
         # TODO: add color mapping for single channel images
         if isinstance(im, np.matrix):
             im = np.array(im)
+
         if im.shape[2] == 1:
+            # L
             im = np.repeat(im, 3, axis=2)
-        elif im.shape[2] == 2:
+        elif im.shape[2] == 2 and self.has_alpha and self.blend_alpha:
+            # LA
+            im = self.blend(im[:, :, 0:1], im[:, :, 1:2])
+            if im.shape[2] == 1:
+                im = np.repeat(im, 3, axis=2)
+        elif im.shape[2] == 2 and self.has_alpha and not self.blend_alpha:
+            # discard A from LA
+            im = np.repeat(im[:, :, 0:1], 3, axis=2)
+        elif im.shape[2] == 2 and not self.has_alpha:
+            # RG -> RGB
             im = np.concatenate((im, np.zeros((im.shape[0], im.shape[1], 1), dtype=im.dtype)), axis=2)
+        elif im.shape[2] == 3:
+            # RGB
+            pass
+        elif im.shape[2] == 4 and self.has_alpha and self.blend_alhpa:
+            # RGBA
+            im = self.blend(im[:, :, :3], im[:, :, 3:4])
+        elif im.shape[2] == 4 and self.has_alpha and not self.blend_alhpa:
+            # discard A from RGBA
+            im = im[:, :, :3]
         elif im.shape[2] != 3:
-            # project to RGB
+            # project spectral to RGB
             if colour is None:
-                raise NotImplemented('please install the colour-science package')
+                raise NotImplemented('please install the colour-science package (pip install colour-science)')
 
             wl_range = self.spec_wl1 - self.spec_wl0
             spec_shape = colour.SpectralShape(self.spec_wl0, self.spec_wl1, wl_range / (im.shape[2] - 1))
@@ -862,8 +932,35 @@ class IV(QMainWindow):
             self.fig.canvas.draw()
             self.setWindowTitle('iv ' + self.timestamp + ' %d / %d' % (self.imind + 1, self.nims))
 
-    def _update_info(self):
-        self.uiLabelInfo.setText(('image: %d / %d\nimage size:\n' % (self.imind + 1, self.nims)) + str(self.get_img().shape))
+    def _update_info(self, pixel=None):
+        if pixel is not None:
+            if len(pixel['value']) <= 3:
+                # short pixel vectors are split at spaces and broken with newlines
+                tmp = str(pixel['value'])
+                while '  ' in tmp:
+                    tmp = tmp.replace('  ', ' ')
+                tmp = tmp.replace('[ ', '[').replace(' ]', ']')
+                tmp = '\n'.join(tmp.split(' '))
+            else:
+                # longer (probably spectral ones) are broken at fixed lengths
+                lines = ['']
+                for ind, val in enumerate(pixel['value']):
+                    last = ind == len(pixel['value']) - 1
+                    if len(lines) == 1:
+                        max_len = 21
+                    else:
+                        max_len = 28
+                    if len(lines[-1]) < max_len:
+                        lines[-1] += ('%.2f' if last else '%.2f, ') % val
+                    else:
+                        lines.append(('%.2f' if last else '%.2f, ') % val)
+                tmp = '[' + '\n'.join(lines) + ']'
+            pixel = '\n(%d,%d): %s' % (pixel['x'], pixel['y'], tmp)
+        else:
+            pixel = ' \n \n '
+        size = str(self.get_img().shape)
+        size = size.replace(', ', 'x')
+        self.uiLabelInfo.setText('img: %d/%d, %s%s' % (self.imind + 1, self.nims, size, pixel))
 
     def _invert_y(self):
         try:
@@ -922,6 +1019,12 @@ class IV(QMainWindow):
             self.y_start += (delta_y - self.prev_delta_y)
             self.prev_delta_x = delta_x
             self.prev_delta_y = delta_y
+        elif event.inaxes:
+            im = self.get_img(tonemap=False, decorate=False)
+            x = np.maximum(0, np.minimum(im.shape[1] - 1, int(event.xdata + 0.5)))
+            y = np.maximum(0, np.minimum(im.shape[0] - 1, int(event.ydata + 0.5)))
+            pixel = {'value': im[y, x], 'x': x, 'y': y}
+            self._update_info(pixel)
 
     def keyPressEvent(self, event: QtGui.QKeyEvent) -> None:
         key = event.key()
@@ -1040,7 +1143,7 @@ class IV(QMainWindow):
         c = QApplication.clipboard()
         c.setImage(im)
 
-    def save(self, ofname=None, zoomed=False, canvas=True):
+    def save(self, ofname=None, zoomed=False, canvas=False, animation=False):
         try:
             if ofname is None:
                 dialog = QFileDialog()
@@ -1050,8 +1153,10 @@ class IV(QMainWindow):
             if ofname is None or not len(ofname):
                 return
             self.ofname = ofname
-            im = self.get_img()
+            if os.path.splitext(ofname)[1].lower() in ['.gif', '.webp', '.mp4']:
+                animation = True
             if zoomed:
+                im = self.get_img()
                 h, w = im.shape[:2]
                 limits = self.ax.axis()
                 x0 = np.max([0, int(limits[0] + 0.5)])
@@ -1072,6 +1177,14 @@ class IV(QMainWindow):
                 width_axes = extent.width
                 height_axes = extent.height
                 image = image[: int(height_axes), : int(width_axes), :]
+            elif animation:
+                from pysmtb.utils import write_video
+                ims = self.get_imgs(tonemap=True, decorate=False)
+                if os.path.splitext(ofname)[1].lower() not in ['.webp', '.mp4', '.gif']:
+                    print('unexpected file extension: %s' % os.path.splitext(ofname)[1].lower())
+                else:
+                    write_video(filename=ofname, frames=ims)
+                return
             else:
                 image = np.array(self.ih.get_array())
             imageio.imwrite(ofname, image)
